@@ -4,11 +4,14 @@ use strict;
 use warnings;
 use autodie;
 use feature qw(say);
+use Convert::Pheno::Default qw(get_defaults);
 use Convert::Pheno::Mapping;
 use Exporter 'import';
 our @EXPORT =
   qw(do_omop2bff $omop_version $omop_main_table @omop_array_tables @omop_essential_tables @stream_ram_memory_tables);
 
+my $DEFAULT = get_defaults();
+
 use constant DEVEL_MODE => 0;
 
 our $omop_version    = 'v5.4';
@@ -53,6 +56,7 @@ our $omop_main_table = {
     ]
 };
 
+# NB: Direct export w/o encapsulation in subroutine
 our @omop_array_tables = qw(
   MEASUREMENT
   OBSERVATION
@@ -62,6 +66,7 @@ our @omop_array_tables = qw(
   VISIT_OCCURRENCE
 );
 
+# NB: Direct export w/o encapsulation in subroutine
 our @omop_essential_tables = qw(
   CONCEPT
   CONDITION_OCCURRENCE
@@ -73,6 +78,7 @@ our @omop_essential_tables = qw(
   VISIT_OCCURRENCE
 );
 
+# NB: Direct export w/o encapsulation in subroutine
 our @stream_ram_memory_tables = qw/CONCEPT PERSON VISIT_OCCURRENCE/;
 
 ##############
@@ -88,9 +94,6 @@ sub do_omop2bff {
     my $ohdsi_dic = $self->{data_ohdsi_dic};
     my $sth       = $self->{sth};
 
-    # Default values to be used accross the module
-    my %default = ( duration => 'P0Y' );
-
     ####################################
     # START MAPPING TO BEACON V2 TERMS #
     ####################################
@@ -209,7 +212,7 @@ sub do_omop2bff {
     # ethnicity
     # =========
 
-    $individual->{ethnicity} = map_ontology(
+    $individual->{ethnicity} = map_ontology_term(
         {
             query => $person->{race_source_value}
             ,    # not getting it from *_concept_id
@@ -281,7 +284,7 @@ sub do_omop2bff {
             };
 
             $exposure->{date}     = $field->{observation_date};
-            $exposure->{duration} = $default{duration};
+            $exposure->{duration} = $DEFAULT->{duration_OMOP};
 
             # _info
             $exposure->{_info}{$table}{OMOP_columns} = $field;
@@ -319,7 +322,7 @@ sub do_omop2bff {
     # geographicOrigin
     # ================
 
-    $individual->{geographicOrigin} = map_ontology(
+    $individual->{geographicOrigin} = map_ontology_term(
         {
             query    => $person->{ethnicity_source_value},
             column   => 'label',
@@ -701,7 +704,7 @@ sub do_omop2bff {
     );
 
     # $sex = {id, label), we need to use 'label'
-    $individual->{sex} = map_ontology(
+    $individual->{sex} = map_ontology_term(
         {
             query    => $sex->{label},
             column   => 'label',
